From dc6800f8cc6f258b972c5fdec94b7585fc6230ca Mon Sep 17 00:00:00 2001
From: Erich Hoover <ehoover@mines.edu>
Date: Wed, 8 May 2013 15:19:09 -0600
Subject: iphlpapi: Implement CancelIPChangeNotify.

---
 dlls/iphlpapi/iphlpapi_main.c  |   19 ++++++++++++++++---
 dlls/iphlpapi/tests/iphlpapi.c |   12 ++++++++++--
 2 files changed, 26 insertions(+), 5 deletions(-)

diff --git a/dlls/iphlpapi/iphlpapi_main.c b/dlls/iphlpapi/iphlpapi_main.c
index 7fa1655..c71d359 100644
--- a/dlls/iphlpapi/iphlpapi_main.c
+++ b/dlls/iphlpapi/iphlpapi_main.c
@@ -220,8 +220,15 @@ DWORD WINAPI AllocateAndGetIpAddrTableFromStack(PMIB_IPADDRTABLE *ppIpAddrTable,
  */
 BOOL WINAPI CancelIPChangeNotify(LPOVERLAPPED overlapped)
 {
-  FIXME("(overlapped %p): stub\n", overlapped);
-  return FALSE;
+    IO_STATUS_BLOCK *iosb = (IO_STATUS_BLOCK *) overlapped;
+    HANDLE handle;
+
+    TRACE("(overlapped %p)\n", overlapped);
+
+    if (!iosb || iosb->u.Status != STATUS_PENDING)
+        return FALSE;
+    handle = (HANDLE) overlapped->u.Pointer;
+    return CloseHandle( handle );
 }
 
 
@@ -2061,6 +2068,11 @@ static NTSTATUS IPHLPAPI_NotifyAddrChange( void *arg, IO_STATUS_BLOCK *iosb, NTS
     size_t len;
     int fd;
 
+    if (status == STATUS_HANDLES_CLOSED)
+    {
+        status = STATUS_CANCELLED;
+        goto done;
+    }
     if (status != STATUS_ALERTED)
         return status;
     if ((status = wine_server_handle_to_fd( h, FILE_READ_DATA, &fd, NULL ) ))
@@ -2071,13 +2083,13 @@ static NTSTATUS IPHLPAPI_NotifyAddrChange( void *arg, IO_STATUS_BLOCK *iosb, NTS
         goto done;
     }
     status = STATUS_SUCCESS;
+done:
     if (iosb)
     {
         iosb->u.Status = status;
         if (overlapped->hEvent)
             SetEvent( overlapped->hEvent );
     }
-done:
     wine_server_release_fd( h, fd );
     return status;
 }
@@ -2153,6 +2165,7 @@ DWORD WINAPI NotifyAddrChange(PHANDLE handle, LPOVERLAPPED overlapped)
     }
     /* for overlapped operation wait for the server to inform us that it's time to read data */
     *handle = h;
+    overlapped->u.Pointer = h;
     SERVER_START_REQ( register_async )
     {
         req->type           = ASYNC_TYPE_READ;
diff --git a/dlls/iphlpapi/tests/iphlpapi.c b/dlls/iphlpapi/tests/iphlpapi.c
index 684b1ec..336bc77 100644
--- a/dlls/iphlpapi/tests/iphlpapi.c
+++ b/dlls/iphlpapi/tests/iphlpapi.c
@@ -35,6 +35,8 @@
  */
 
 #include <stdarg.h>
+#include "ntstatus.h"
+#define WIN32_NO_STATUS
 #include "winsock2.h"
 #include "windef.h"
 #include "winbase.h"
@@ -1054,7 +1056,7 @@ static void testNotifyAddrChange(void)
     ret = GetLastError();
     todo_wine ok(ret == ERROR_IO_PENDING, "GetLastError returned %d, expected ERROR_IO_PENDING\n", ret);
     success = pCancelIPChangeNotify(&overlapped);
-    todo_wine ok(success == TRUE, "CancelIPChangeNotify returned FALSE, expected TRUE\n");
+    ok(success == TRUE, "CancelIPChangeNotify returned FALSE, expected TRUE\n");
 
     ZeroMemory(&overlapped, sizeof(overlapped));
     success = pCancelIPChangeNotify(&overlapped);
@@ -1071,7 +1073,13 @@ static void testNotifyAddrChange(void)
     ret = GetLastError();
     ok(ret == ERROR_IO_INCOMPLETE, "GetLastError returned %d, expected ERROR_IO_INCOMPLETE\n", ret);
     success = pCancelIPChangeNotify(&overlapped);
-    todo_wine ok(success == TRUE, "CancelIPChangeNotify returned FALSE, expected TRUE\n");
+    ok(success == TRUE, "CancelIPChangeNotify returned FALSE, expected TRUE\n");
+    ok(overlapped.Internal == STATUS_CANCELLED, "Overlapped IO returned %x, expected %x\n",
+                                                (int)overlapped.Internal, STATUS_CANCELLED);
+    success = GetOverlappedResult(handle, &overlapped, &bytes, FALSE);
+    ok(success == FALSE, "GetOverlappedResult returned TRUE, expected FALSE\n");
+    ret = GetLastError();
+    ok(ret == ERROR_OPERATION_ABORTED, "GetLastError returned %d, expected ERROR_OPERATION_ABORTED\n", ret);
 
     if (winetest_interactive)
     {
-- 
1.7.9.5

